/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "SIF.h"
#include "BSLibGlobal.h"
#include "ArgumentData.h"
#include "Variable.h"

using namespace Scratch;

namespace BromScript{
	namespace Libaries{
		namespace Global{
			BS_FUNCTION(Call){
				if (!args->CheckType(0, Variable::VariableType::Function, true)) return null;

				Function* tryfunc = args->GetFunction(0);
				List<Variable*> vars;

				for(int i = 1; i < args->Count; i++)
					vars.Add(args->GetVariable(i));

				tryfunc->Run(&vars);

				return Converter::ToVariable(!args->ErrorOccured());
			}

			BS_FUNCTION(Print){
				CString out;
				for (int i = 0; i < args->Count; i++){
					if (i > 0)
						out += '\t';

					switch(args->GetType(i)){
					case Variable::VariableType::String: out += args->GetString(i); break;
					case Variable::VariableType::Number: out += CString(args->GetNumber(i)); break;
					case Variable::VariableType::Function: out += "Function"; break;
					case Variable::VariableType::Bool: out += args->GetBool(i) ? "True" : "False"; break;
					case Variable::VariableType::Table: out += CString("{table count=") + CString((double)args->GetTable(i)->Count) + "}"; break;
					case Variable::VariableType::Null: out += "NULL\n"; break;
					default:
						bool found = false;
						for (int i2 = 0; i2 < bsi->RegisteredUserdataTypes.Count; i2++){
							if (args->GetType(i) == bsi->RegisteredUserdataTypes[i2]->TypeID){
								found = true;
								out += CString("Userdata{") + bsi->RegisteredUserdataTypes[i2]->Name + "}";
							}
						}

						if (!found)
							out += "Userdata{Unknown}";
						break;
					}
				}

				out += "\n";
				printf(out);
				return null;
			}

			BS_FUNCTION(ToString){
				CString out;
				switch(args->GetType(0)){
				case Variable::VariableType::String: out = args->GetString(0); break;
				case Variable::VariableType::Number: out = CString(args->GetNumber(0)); break;
				case Variable::VariableType::Function: out = "Function"; break;
				case Variable::VariableType::Bool: out = args->GetBool(0) ? "True" : "False"; break;
				case Variable::VariableType::Table: out = CString("{table count=") + CString((double)args->GetTable(0)->Count) + "}"; break;
				case Variable::VariableType::Null: out = "NULL\n"; break;
				default:
					bool found = false;
					for (int i2 = 0; i2 < bsi->RegisteredUserdataTypes.Count; i2++){
						if (args->GetType(0) == bsi->RegisteredUserdataTypes[i2]->TypeID){
							found = true;
							out = CString("Userdata{") + bsi->RegisteredUserdataTypes[i2]->Name + "}";
						}
					}

					if (!found)
						out = "Userdata{Unknown}";
					break;
				}

				return Converter::ToVariable(out);
			}

			BS_FUNCTION(TypeOf){
				return Converter::ToVariable(Converter::TypeToString(bsi, args->GetType(0)));
			}

			BS_FUNCTION(Include){
				if (!args->CheckType(0, Variable::VariableType::String, true)) return null;

				bsi->DoFile(args->GetString(0));
				return null;
			}

			BS_FUNCTION(IncludeDll){
				if (!args->CheckType(0, Variable::VariableType::String, true)) return null;

				bsi->DoDLL(args->GetString(0));
				return null;
			}

			BS_FUNCTION(SetReadOnly){
				if (args->Count != 1){
					args->Error("Expected variable, got nothing!");
					return null;
				}
				
				args->GetVariable(0)->ReadOnly = true;
				return null;
			}

			BS_FUNCTION(Error){
				if (!args->CheckType(0, Variable::VariableType::String, true)) return null;

				args->Error(args->GetString(0));
				return null;
			}
		}
	}
}