/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "SIF.h"
#include "BSLibString.h"
#include "ArgumentData.h"
#include "Variable.h"

using namespace Scratch;

namespace BromScript{
	namespace Libaries{
		namespace String{
			BS_FUNCTION(FromChar){
				if (!args->CheckType(1, Variable::VariableType::Number, true)) return null;
				return Converter::ToVariable(CString((char)args->GetNumber(1)));
			}
			
			BS_FUNCTION(ToChar){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				CString str = args->GetString(1);
				if (str.Size() == 0){
					args->Error("Cannot convert 0 sized string to char");
					return null;
				}

				return Converter::ToVariable((int)str[0]);
			}
			
			BS_FUNCTION(Split){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				if (!args->CheckType(2, Variable::VariableType::String, true)) return null;

				CStackArray<CString> arr;
				CString(args->GetString(1)).Split(args->GetString(2), arr);

				Table* tbl = new Table(args->BromScript);
				for (int i = 0; i < arr.Count(); i++)
					tbl->Set(CString((double)i), Converter::ToVariable(arr[i]));

				return Converter::ToVariable(tbl);
			}

			BS_FUNCTION(Sub){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				if (!args->CheckType(2, Variable::VariableType::Number, true)) return null;

				if (args->Count == 4 && !args->CheckType(3, Variable::VariableType::Number, true)) return null;

				if (args->Count == 4) return Converter::ToVariable(CString(args->GetString(1)).Substring(args->GetNumber(2), args->GetNumber(3)));
				else return Converter::ToVariable(CString(args->GetString(1)).Substring(args->GetNumber(2)));
			}

			BS_FUNCTION(IndexOf){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				if (args->Count < 3){
					args->Error("Expected 2(or 3) arguments (String, Char/String, [Number])!");
					return null;
				}

				if (args->CheckType(2, Variable::VariableType::String)){
					return Converter::ToVariable(CString(args->GetString(1)).IndexOf(args->GetString(2), args->GetNumber(3)));
				}else if (args->CheckType(2, Variable::VariableType::Number)){
					return Converter::ToVariable(CString(args->GetString(1)).IndexOf((char)args->GetNumber(2), args->GetNumber(3)));
				}else{
					args->Error("Expected 2(or 3) arguments (String, Char/String, [Number])!");
					return null;
				}
			}

			BS_FUNCTION(LastIndexOf){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				if (args->Count < 3){
					args->Error("Expected 2(or 3) arguments (String, Char/String, [Number])!");
					return null;
				}

				CString str = args->GetString(1);
				int spos = str.Size() - 1;
				if (args->Count == 4){
					if (!args->CheckType(3, Variable::VariableType::Number, true)) return null;
					spos = args->GetNumber(3);
				}

				if (args->CheckType(2, Variable::VariableType::String)){
					return Converter::ToVariable(str.IndexOf(args->GetString(2), spos));
				}else if (args->CheckType(2, Variable::VariableType::Number)){
					return Converter::ToVariable(str.IndexOf((char)args->GetNumber(2), spos));
				}else{
					args->Error("Expected 2(or 3) arguments (String, Char/String, [Number])!");
					return null;
				}
			}

			BS_FUNCTION(Left){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				if (!args->CheckType(2, Variable::VariableType::Number, true)) return null;
				return Converter::ToVariable(CString(args->GetString(1)).Substring(args->GetNumber(2)));
			}

			BS_FUNCTION(Right){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				if (!args->CheckType(2, Variable::VariableType::Number, true)) return null;

				CString str = args->GetString(1);
				int len = str.Size() - 1;
				int offset = args->GetNumber(2);

				return Converter::ToVariable(str.Substring(len - offset, offset));
			}

			BS_FUNCTION(Trim){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;

				if (args->Count == 2){
					return Converter::ToVariable(CString(args->GetString(1)).Trim());
				}else{
					if (!args->CheckType(2, Variable::VariableType::String, true)) return null;

					CString str = args->GetString(1);
					CString chars = args->GetString(2);
					int len = chars.Size();
					for (int i = 0; i < len; i++) str.Trim(chars[i]);

					return Converter::ToVariable(str);
				}
			}

			BS_FUNCTION(TrimLeft){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;

				if (args->Count == 2){
					return Converter::ToVariable(CString(args->GetString(1)).TrimLeft());
				}else{
					if (!args->CheckType(2, Variable::VariableType::String, true)) return null;

					CString str = args->GetString(1);
					CString chars = args->GetString(2);
					int len = chars.Size();
					for (int i = 0; i < len; i++) str.TrimLeft(chars[i]);

					return Converter::ToVariable(str);
				}
			}

			BS_FUNCTION(TrimRight){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;

				if (args->Count == 2){
					return Converter::ToVariable(CString(args->GetString(1)).TrimRight());
				}else{
					if (!args->CheckType(2, Variable::VariableType::String, true)) return null;

					CString str = args->GetString(1);
					CString chars = args->GetString(2);
					int len = chars.Size();
					for (int i = 0; i < len; i++) str.TrimRight(chars[i]);

					return Converter::ToVariable(str);
				}
			}

			BS_FUNCTION(Replace){
				if (!args->CheckType(1, Variable::VariableType::String, true)) return null;
				if (!args->CheckType(2, Variable::VariableType::String, true)) return null;
				if (!args->CheckType(3, Variable::VariableType::String, true)) return null;

				return Converter::ToVariable(CString(args->GetString(1)).Replace(args->GetString(2), args->GetString(3)));
			}
		}
	}
}